#include "macierz.h"
#include <stdlib.h>
#include <stdio.h>

/* Funkcja tworzy macierz (alokuje pamiec) o podanej liczbie wierszy 'w' i kolumn 'k'. 
 * Argumenty:
 *   'w' - liczba wierszy
 *   'k' - liczba kolumn
 * Wartoscia zwracana jest wskaznik do macierzy (adres poczatku tablicy zawierajacej adresy kolejnych wierszy macierzy).  */
float **utworz_macierz(int w, int k)
{
   float **m;
   int i;
   m = (float **) malloc(w * sizeof(float*));
   if(m == NULL) return NULL;
   for(i=0 ; i<w ; i++)
   {
      m[i] = (float *) malloc(k *sizeof(float));
      if(m[i]==NULL) 
      {
         /*  sprzatanie juz przydzielonej pamieci */
         zwolnij_macierz(m,i);
         return NULL;
      }
   }
   return m;
}
/*  Funkcja wyswietla macierz na standardowym wyjsciu (na ekranie).
 *  Argumenty:
 *    'm' - wskaznik do macierzy (adrese poczatku tablicy zawierajacej adresy do kolejnych wierszy macierzy) 
 *    'w' - liczba wierszymacierzy
 *    'k' - liczba kolumn macierzy
 */
void wyswietl_macierz(float **m,int w,int k)
{
   int i,j;
   for(i=0 ; i<w ; i++)
   {
      for(j=0 ; j<k ; j++)
         printf(" %6.1f", *(*(m+i)+j) );
      printf("\n");
   }
}

/*  Funkcja wczytuje elementy macierzy ze standardowego wejscia (z klawiatury). 
 *  Argementy:
 *     'm' - adres (wskaznik) do macierzy, 
 *     'w' -liczba wierszy, 
 *     'k' - liczba kolumn.  
 * Wartoscia zwracana jest adres do macierzy o zmodyfikowanych wartosciach. */
float** wczytaj_macierz(float **m,int w,int k)
{
   int i,j;
   for(i=0 ; i<w ; i++)
   {
      for(j=0 ; j<k ; j++)
      {
         printf(" [%d,%d] = ", i,j);
         scanf("%f",*(m+i)+j); 
      }
   }
   return m;
}


/* Funkcja zwalnia pamiec zajeta przez macierz 'm'.
 * Argumenty:
 *   'm' - wskaznik do macierzy
 *   'w' - liczba wierszy
 **/
void zwolnij_macierz(float **m, int w)
{
   while(--w > 0) free(*(m+w));
   free(m);
}

/* Funkcja odczytuje macierz z pliku. 
 * Argumenty:
 *   'p' - wskaznik do pliku (FILE) tekstowego otworzonego do odczytu.
 *   'w' - wskaznik do zmiennej w ktorej funkcja umiesci liczbe wierszy 
 *   'k' - wskaznik do zmiennej w ktorej funkcja umiesci liczbe kolumn
 * Zakladamy, ze na poczatku pliku znajduja sie dwie liczby calkowite okreslajace wymiary macierzy (liczba wierszy i liczba kolumn) 
 * a nastepnie ciag liczb rzeczywistych, czyli elementy macierzy.
 * Wartoscia zwracana jest wskaznik float** do macierzy odczytanej z pliku lub wartosc NULL jezeli nastapil blad alokacji pamieci. 
   */
float **wczytaj_z_pliku(FILE *p,int *w,int *k)
{
   int i,j;
   float **m;

   fscanf(p,"%d %d",w,k);
   m=utworz_macierz(*w,*k);
   if(m==NULL) return NULL;
   for(i=0 ; i<*w ; i++)
   {
      for(j=0 ; j<*k ; j++)
      {
         fscanf(p,"%f",*(m+i)+j); 
      }
   }
   return m;
}

/* Funkcja zapisuje macierz do pliku.
 * Argumenty:
 *  'p' - wskaznik do pliku tekstowego otworzonego do zapisu
 *  'm' - wskaznik do macierzy
 *  'w' - liczba wierszy
 *  'k' - liczba kolumn
 * Macierz zapisywana jest w pojedynczej linii na poczatku umieszczane sa dwie liczby calkowite okreslajace ilosc wierszy i kolumn macierzy,  
 * nastepnie w pliku umieszczane sa  wszystkie elementy macierzy z dokladnoscia do 1 miejsca po przecinku.
 */
void zapisz_do_pliku(FILE *p,float **m,int w,int k)
{
   int i,j;
   fprintf(p,"%d %d",w,k);   

   for(i=0 ; i<w ; i++)
   {
      for(j=0 ; j<k ; j++)
         fprintf(p," %.1f", *(*(m+i)+j) );
   }
   fprintf(p,"\n");
}
